package cz.mendelu.dnaAnalyser.sequence.annotation;

import com.fasterxml.jackson.annotation.JsonIgnore;
import cz.mendelu.dnaAnalyser.sequence.Sequence;
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.Feature;
import cz.mendelu.dnaAnalyser.user.User;
import cz.mendelu.dnaAnalyser.utils.analyse.data.Localised;
import cz.mendelu.dnaAnalyser.utils.analyse.model.Analyse;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import cz.mendelu.dnaAnalyser.utils.model.Tagged;
import lombok.*;
import org.hibernate.annotations.CreationTimestamp;

import javax.persistence.*;
import java.util.*;

import static javax.persistence.CascadeType.ALL;
import static javax.persistence.FetchType.LAZY;

@Data
@Entity
@ToString(callSuper = true)
@NoArgsConstructor(access = AccessLevel.PROTECTED)
@Table(name = "annotations")
public class Annotation implements Identifiable, Tagged {
    @Builder
    private Annotation(
            UUID id, Date created, Sequence sequence, User owner, String type, Integer start, Integer stop) {
        this.id = (id != null) ? id : UUID.randomUUID();
        this.type = type;
        this.start = start;
        this.stop = stop;
        this.created = created;
        this.sequence = sequence;
        this.owner = owner;

    }

    @Id
    protected UUID id;

    @Column(name = "type")
    private String type;

    @Column(name = "start")
    private Integer start;

    @Column(name = "stop")
    private Integer stop;

    @Temporal( TemporalType.TIMESTAMP )
    @CreationTimestamp
    protected Date created;

    @ManyToOne(cascade = CascadeType.DETACH)
    @JsonIgnore
    protected Sequence sequence;

    @ManyToOne(cascade = CascadeType.DETACH)
    @JsonIgnore
    protected User owner;

    public UUID getSequenceId() {
        return sequence.getId();
    }

    @ElementCollection
    private Set<String> tags;
}
