package cz.mendelu.dnaAnalyser.sequence.stream;

import cz.mendelu.dnaAnalyser.sequence.Sequence;

import java.util.Optional;


/**
 * Window Stream Service.
 * It contains a method for work with DNA sequence data as with Stream.
 */
public interface StreamService {

    /**
     * Open a new {@link WindowStream} for the given sequence.
     * <p>
     *     The sequence will have the data ready to be used only if they have already been imported previously.
     *     But to process a large amount of recently imported data might take same time.
     *     This is the reason why this method returns you the {@link Optional} intend of the {@link WindowStream} directly.
     * </p>
     * <p>
     *     The WindowStream will progressively move through the sequence from one Nucleotide to the next Nucleotide.
     * <pre>
     * open(ACTGACTG, 4)
     *      |  |
     *      ACTG
     *       CTGA
     *        TGAC
     *         GACT
     *          ACTG
     * </pre>
     * </p>
     * @param sequence for the data to be opened.
     * @param windowsSize size of windows
     * @return {@link Optional} object of WindowStream.
     */
    // TODO Replace optional with Future???
    Optional<WindowStream> open(Sequence sequence, int windowsSize);
}
