package cz.mendelu.dnaAnalyser.sequence

import cz.mendelu.dnaAnalyser.sequence.Sequence
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData
import spock.lang.Specification

class SequenceDataTest extends Specification {

    UUID sequenceId
    UUID bufferId
    Sequence sequence

    def setup() {
        sequenceId = UUID.randomUUID()
        bufferId = UUID.randomUUID()
        sequence = new Sequence(
                id: sequenceId,
                bufferId: bufferId
        )
    }

    def "test getLength"() {
        when:
        def sequenceBuffer = SequenceData.wrap(sequence, 'ACTG'.bytes)

        then:
        sequenceBuffer.length == 4
    }

    def "test wrap Sequence Buffer without Id"() {
        when:
        SequenceData.wrap(null, 'ACTG'.bytes)

        then:
        thrown(java.lang.AssertionError)
    }

    def "test toString data with range"() {
        given:
        def sequenceBuffer = SequenceData.wrap(sequence, 'ACTG'.bytes)

        when:
        def toString = sequenceBuffer.toString(1,2)

        then:
        toString == 'CT'
    }

    def "test toString data with length over the buffer"() {
        given:
        def sequenceBuffer = SequenceData.wrap(sequence, 'ACTG'.bytes)

        when:
        def toString = sequenceBuffer.toString(2,4)

        then:
        toString == 'TG'
    }

    def "test toString data with position after the buffer"() {
        given:
        def sequenceBuffer = SequenceData.wrap(sequence, 'ACTG'.bytes)

        when:
        def toString = sequenceBuffer.toString(6,4)

        then:
        toString == ''
    }

    def "test toString"() {
        when:
        def sequenceBuffer = SequenceData.wrap(sequence, 'ACTG'.bytes)

        then:
        sequenceBuffer.toString() != null
    }
}
