package cz.mendelu.dnaAnalyser.sequence


import cz.mendelu.dnaAnalyser.sequence.data.SequenceData
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide
import spock.lang.Specification


class SequenceData_TrimBuffer_Test extends Specification {

    Sequence sequence
    SequenceData data

    def setup() {
        sequence = new Sequence()
        data = SequenceData.wrap(sequence, 'CGTTGATCAACG'.bytes)
    }

    def "test get method"() {
        when:
        def buffer = data.getBuffer(3,4)

        then:
        buffer.get(0) == Nucleotide.T
        buffer.get(1) == Nucleotide.G
        buffer.get(2) == Nucleotide.A
        buffer.get(3) == Nucleotide.T
    }

    def "test get method on revert"() {
        when:
        def buffer = data.getBuffer(3,4).revert()

        then:
        buffer.get(0) == Nucleotide.T
        buffer.get(1) == Nucleotide.A
        buffer.get(2) == Nucleotide.G
        buffer.get(3) == Nucleotide.T
    }

    def "test get method after bound"() {
        given:
        def buffer = data.getBuffer(3,4)
        when:
        buffer.get(4) == Nucleotide.T

        then:
        thrown(IndexOutOfBoundsException)
    }

    def "test get method after bound on revert"() {
        given:
        def buffer = data.getBuffer(3,4).revert()
        when:
        buffer.get(4) == Nucleotide.T

        then:
        thrown(IndexOutOfBoundsException)
    }

    def "test next"() {
        when:
        def buffer = data.getBuffer(3,4)

        then:
        buffer.next() == Nucleotide.T
        buffer.next() == Nucleotide.G
        buffer.next() == Nucleotide.A
        buffer.next() == Nucleotide.T
        buffer.hasNext() == false
    }

    def "test next on revert"() {
        when:
        def buffer = data.getBuffer(3,4).revert()

        then:
        buffer.next() == Nucleotide.T
        buffer.next() == Nucleotide.A
        buffer.next() == Nucleotide.G
        buffer.next() == Nucleotide.T
        buffer.hasNext() == false
    }

    def "test length"() {
        when:
        def buffer = data.getBuffer(3,4)

        then:
        buffer.length == 4
    }

    def "test length on revert"() {
        when:
        def buffer = data.getBuffer(3,4).revert()

        then:
        buffer.length == 4
    }

    def "test get position after 2 steps"() {
        given:
        def buffer = data.getBuffer(3,4)

        when:
        buffer.next()
        buffer.next()

        then:
        buffer.position == 3 + 2
    }
}
