package cz.mendelu.dnaAnalyser.sequence.model;

import cz.mendelu.dnaAnalyser.sequence.NucleicBuffer;
import cz.mendelu.dnaAnalyser.sequence.SequenceType;
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide;
import org.apache.commons.lang3.ArrayUtils;

public class TestNucleicBuffer implements NucleicBuffer {

    private final byte[] data;
    private int position = 0;
    private SequenceType type;

    public TestNucleicBuffer(byte[] data) {
        this.data = data;
    }

    public TestNucleicBuffer(String data) {
        this(data.getBytes());
    }

    @Override
    public NucleicBuffer revert() {
        byte[] copy = ArrayUtils.clone(data);
        ArrayUtils.reverse(copy);
        return new TestNucleicBuffer(copy);
    }

    @Override
    public boolean isCyclic() {
        return false;
    }

    public int getAbsoluteStartPosition() {
        return 0;
    }

    @Override
    public int getLength() {
        return data.length;
    }

    @Override
    public NucleicBuffer getSubBuffer(int position, int length) {
       return null;
    }

    @Override
    public int getPosition() {
        return position;
    }

    @Override
    public Nucleotide get(int pos) {
        return Nucleotide.get(data[position + pos]);
    }

    @Override
    public SequenceType getType() {
        if(ArrayUtils.contains(data, (byte)'U')){
            return SequenceType.RNA;
        } else
        return SequenceType.DNA;
    }

    @Override
    public String toPlain() {
        return new String(data);
    }

    @Override
    public boolean hasNext() {
        return position <  data.length;
    }

    @Override
    public Nucleotide next() {
        return Nucleotide.get(data[position++]);
    }

    public void setType(SequenceType type){
        this.type = type;
    }
}